package org.puremvc.as3.multicore.utilities.fabrication.global
{
	import flash.events.IEventDispatcher;
	
	import mx.core.IMXMLObject;
	import mx.events.FlexEvent;
	
	import org.as3commons.reflect.Type;
	import org.puremvc.as3.multicore.interfaces.IFacade;
	import org.puremvc.as3.multicore.utilities.fabrication.Fabrication;
	import org.puremvc.as3.multicore.utilities.fabrication.injection.FabMetadataTag;
	import org.puremvc.as3.multicore.utilities.fabrication.injection.MediatorInjectorProcessor;
	import org.puremvc.as3.multicore.utilities.fabrication.injection.ProxyInjectorProcessor;

	public class FabricationConfig implements IMXMLObject
	{
		
		/**
		 * fabrication version
		 */ 
		public static const version:String = "0.0.1"
		
		/**
		 * current application fabrication instance
		 */ 
		public static var appfabr_IFacade:IFacade;
		
		/**
		 * inject enabled
		 */ 
		public static var inject_enabledBool:Boolean = true;
		
		/**
		 * default metadata
		 */ 
		public static var metaDatas:Array = [FabMetadataTag.INJECT_PROXY,FabMetadataTag.INJECT_MEDIATOR];
		
		/**
		 * default processor
		 */ 
		public static var processors:Array = [ new ProxyInjectorProcessor() , new MediatorInjectorProcessor() ] 
			
		/**
		 * default custom filter package
		 * need cache in filterPackage
		 */ 
		public static var filterPackages:Array = [Fabrication.short_packageName];
		
		/**
		 * custom metaData
		 */ 
		private var _customMetaDatas:Array;
		public function get customMetaDatas():Array
		{
			return _customMetaDatas;
		}
		public function set customMetaDatas(value:Array):void
		{
			_customMetaDatas = value;
			metaDatas = metaDatas.concat(_customMetaDatas);
		}
		
		/**
		 * custom Processors
		 */ 
		private var _customProcessors:Array;
		public function get customProcessors():Array
		{
			return _customProcessors;
		}
		public function set customProcessors(value:Array):void
		{
			_customProcessors = value;
			processors = processors.concat(_customProcessors);
		}
		
		private var _viewPackages:Array;
		public function get viewPackages():Array
		{
			return _viewPackages;
		}
		public function set viewPackages( value:Array ):void
		{
			_viewPackages = value;
			setViewPackages( value );
		}
		
		/**
		 * custom cache filterPackage
		 */ 
		private var _customFilterPackage:Array;
		public function get customFilterPackage():Array
		{
			return _customFilterPackage;
		}
		public function set customFilterPackage(value:Array):void
		{
			_customFilterPackage = value;
			filterPackages = filterPackages.concat(_customFilterPackage);
		}
		
		
		/**
		 * @see mx.core.IMXMLObject#initialized
		 */
		public function initialized( document:Object, id:String ):void
		{
			
			IEventDispatcher( document ).addEventListener( FlexEvent.PREINITIALIZE, handleContainerPreinitialize );
			
		}
		
		private function handleContainerPreinitialize(f:FlexEvent):void
		{
			
			customMetaDatas  		= addCustomMetaDatas()
			customProcessors 		= addCustomProcessors();
			viewPackages     		= addViewPackages();
			customFilterPackage 	= addCustomFilterPackage();
			
			Type.filter_packages = filterPackages;
			
		}
				
		protected function addCustomMetaDatas():Array{
			return [];
		};
		
		protected function addCustomProcessors():Array{
			return [];
		};
		
		protected function addViewPackages():Array{
			return [];
		};
		
		protected function addCustomFilterPackage():Array{
			return [];
		}
		
		/**
		 * view factory
		 */ 
		public var viewFactory:FabricationViewFactory;
		
		/**
		 * Internal setter for <code>viewPackages</code> property.
		 *
		 * @param value An Array of Strings or a single String that will be split on ","
		 */
		protected function setViewPackages( value:* ):void
		{
			_viewPackages = parsePackageValue( value );
			
			if(_viewPackages.length > 0 ){
				viewFactory = new FabricationViewFactory();
			}
		}
		
		/**
		 * Parses a wildcard type package property value into an Array of parsed package names.
		 *
		 * @param value An Array of Strings or a single String that will be split on ","
		 * @returns An Array of package name strings in a common format.
		 */
		protected function parsePackageValue( value:* ):Array
		{
			if( value == null )
			{
				return [];
			}
			else if( value is Array )
			{
				return parsePackageNames( value as Array );
			}
			return [];
		}
		
		/**
		 * Parses an array of package names.
		 * Processes the package names to a common format - removing trailing '.*' wildcard notation.
		 *
		 * @param packageNames The package names to parse.
		 * @returns An Array of the parsed package names.
		 */
		protected function parsePackageNames( packageNames:Array ):Array
		{
			var parsedPackageNames:Array = [];
			
			for each( var packageName:String in packageNames )
			{
				parsedPackageNames.push( parsePackageName( packageName ) );
			}
			
			return parsedPackageNames;
		}
		
		/**
		 * Regular expression to evaluate a 'wildcard' (ex. 'org.swizframework.*') package description.
		 *
		 * Matches: package.*
		 * Captures: package
		 */
		protected static const WILDCARD_PACKAGE:RegExp = /\A(.*)(\.\**)\Z/;
		
		/**
		 * Parse Package Name
		 * Processes the package name to a common format - removing trailing '.*' wildcard notation.
		 *
		 * @param packageName The package name to parse.
		 * @returns The package name with the wildcard notation stripped.
		 */
		protected function parsePackageName( packageName:String ):String
		{
			var match:Object = WILDCARD_PACKAGE.exec( packageName );
			if( match )
				return match[ 1 ];
			
			return packageName;
		}
		
		
			
		
		
	}
}