/**
 * Copyright (C) 2010 Rafał Szemraj.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.puremvc.as3.multicore.utilities.fabrication.injection {
    import org.puremvc.as3.multicore.interfaces.IFacade;
    import org.puremvc.as3.multicore.interfaces.IMediator;
    import org.puremvc.as3.multicore.patterns.facade.Facade;
    import org.puremvc.as3.multicore.utilities.fabrication.global.FabricationConfig;
    import org.puremvc.as3.multicore.utilities.fabrication.interfaces.IInjectProcessor;

    /**
     * Injector implementation for PurMVC mediators
	 * 
     * [InjectMediator(name="mediator")]
	   public var mediator:mediator
     */
    public class MediatorInjectorProcessor implements IInjectProcessor{


        /**
         * @inheritDoc
         */
		public function elementExist(injectionField:InjectionField,facade:IFacade=null):IFacade 
		{
			if( injectionField.elementName != null )
			{
				if(facade == null)
				{
					if(FabricationConfig.appfabr_IFacade.hasMediator(injectionField.elementName)){
						return FabricationConfig.appfabr_IFacade;
					}
					
					var a:Array = Facade.instanceMap;
					for each(var fac:IFacade in a)
					{
						if(fac!=null)
						{
							if(fac.hasMediator(injectionField.elementName)){
								return fac;
							}
						}
					}
				}
				else
				{
					if(facade.hasMediator(injectionField.elementName))
					{
						return facade;
					}
					else
					{
						if(FabricationConfig.appfabr_IFacade.hasMediator(injectionField.elementName)){
							return FabricationConfig.appfabr_IFacade;
						}
					}
				}
			}
			return null;
		}

        /**
         * Returns name of element to be injected. Because here we are looking for registered mediator
         * we can ommit metadata "name" param and try to retrieve mediator by static NAME property of
         * field class
         * @param field field instance to process
         * @param tagName name of injection tag
         */
		public function processor(injectionField:InjectionField,facade:IFacade  ):Object
		{
			
			return facade.retrieveMediator( injectionField.elementName ) as IMediator;
          
		}
    }
}